﻿using C1.TextParser;
using C1_hl7.data.Model;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Text.Json;

namespace C1_hl7.data;

public class HL7Datareader
{
    private const string _templateResource = "C1_hl7.data.Templates.HL7Template.xml";

    internal TemplateBasedExtractor ReadTemplateBasedExtractor()
    {
        var assembly = Assembly.GetExecutingAssembly();
        using Stream stream = assembly.GetManifestResourceStream(_templateResource);
        return new TemplateBasedExtractor(stream);
    }

    public List<ISegment> ReadData(string hl7String)
    {
        // the Extract method expects a stream
        using MemoryStream hl7 = new MemoryStream(Encoding.ASCII.GetBytes(hl7String));
        TemplateBasedExtractor templateBasedExtractor = ReadTemplateBasedExtractor();
        IExtractionResult extractionResult = templateBasedExtractor.Extract(hl7);
        // The final result of the extraction is a json string 
        string json = extractionResult.ToJsonString();

        // deserialize the json string
        HL7Message? messages = JsonSerializer.Deserialize<HL7Message>(json);

        return InterpretExtractedData(messages);
    }

    private List<ISegment> InterpretExtractedData(HL7Message? message)
    {
        List<ISegment> segments = new();
        ISegment nextOfKin = new Segment("Next of Kin", string.Empty, string.Empty);
        ISegment observations = new Segment("Observations", string.Empty, string.Empty);

        foreach (var segment in message.Result.HL7Segment)
        {
            string segmentType = segment.common.Type;
            string[] fields = segment.Fields.Split('|');
            switch (segmentType)
            {
                case "MSH":     // Message header
                    segments.Add(new Segment(segmentType, fields[1], ToDate(fields[4])));
                    break;
                case "EVN":     // Event type
                    segments.Add(new Segment(segmentType, "Encounter", ToDate(fields[1])));
                    break;
                case "PID":     // Patient Identification
                    segments.Add(new Segment(segmentType, ToName(fields[3]), ToDate(fields[5])));
                    break;
                case "PV1":     // Patient Visit
                    segments.Add(new Segment(segmentType, "Dr.", ToPratitionerName(fields[7])));
                    break;
                case "NK1":     // Next of Kin
                    nextOfKin.Subsegments.Add(new Segment(segmentType, ToName(fields[0]), fields[1]));
                    break;
                case "OBX":     // ObservationResult
                    string obs = fields[1].Split('^').Last();
                    observations.Subsegments.Add(new Segment(segmentType, obs, fields[3] + " " + fields[4]));
                    break;
                case "DG1":     // Diagnosis Information
                    segments.Add(new Segment(segmentType, fields[2], fields[4]));
                    break;
            }
        }

        segments.Add(nextOfKin);
        segments.Add(observations);

        return segments;

        string ToName(string humanName)
        {
            string[] nameparts = humanName.Split('^');

            return nameparts.Length switch
            {
                0 => string.Empty,
                1 => nameparts[0],
                _ => $"{nameparts[0]} {nameparts[1]}"
            };
        }

        string ToPratitionerName(string humanName)
        {
            string[] nameparts = humanName.Split('^');

            return nameparts.Length switch
            {
                0 => string.Empty,
                1 => nameparts[0],
                2 => nameparts[1],
                _ => $"{nameparts[2]} {nameparts[3]}"
            };
        }


        string ToDate(string dt)
        {
            if (DateTime.TryParseExact(dt, new string[] { "yyyyMMddHHmm", "yyyyMMdd" }, null, DateTimeStyles.None, out DateTime d))
                return d.ToString();
            else return dt;
        }
    }
}
